window.Gleam.initSearch([{"doc":"qcheck","title":"qcheck","content":"# qcheck\n\nQuickCheck-inspired property-based testing with integrated shrinking for [Gleam](https://gleam.run/).\n\nRather than specifying test cases manually, you describe the invariants that values of a given type must satisfy (&quot;properties&quot;). Then, generators generate lots of values (test cases) on which the properties are checked. Finally, if a value is found for which a given property does not hold, that value is &quot;shrunk&quot; in order to find an nice, informative counter-example that is presented to you.\n\nWhile there are a ton of great articles introducing quickcheck or property-based testing, here are a couple general resources that you may enjoy:\n\n- [An introduction to property based testing](https://fsharpforfunandprofit.com/pbt/)\n- [What is Property Based Testing?](https://hypothesis.works/articles/what-is-property-based-testing/)\n\nYou might also be interested in checking out [this project](https://github.com/mooreryan/gleam_stdlib_testing) that uses qcheck to test Gleam&#39;s stdlib.\n\n## Usage &amp; Examples\n\nSee API docs for detailed usage.\n\n### Basic example\n\nHere is a short example to get you started. It assumes you are using [gleeunit](https://github.com/lpil/gleeunit) to run the tests, but any test runner that reasonably handles panics will do.\n\n```gleam\nimport qcheck\n\npub fn small_positive_or_zero_int__test() {\n  use n &lt;- qcheck.given(qcheck.small_positive_or_zero_int())\n  n + 1 == 1 + n\n}\n\npub fn small_positive_or_zero_int__failures_shrink_to_zero__test() {\n  use n &lt;- qcheck.given(qcheck.small_positive_or_zero_int())\n  n + 1 != 1 + n\n}\n```\n\nThat second example will fail with an error that may look something like this if you are targeting Erlang.\n\n```\n Failures:\n\n  1) examples/basic_example_test.small_positive_or_zero_int__failures_shrink_to_zero__test\n     Failure: &lt;&lt;&quot;TestError[original_value: 3; shrunk_value: 0; shrink_steps: 1; error: property was False;]&quot;&gt;&gt;\n     stacktrace:\n       qcheck_ffi.fail\n```\n\n- `qcheck.given` sets up the test\n  - If a property holds for all generated values, then `qcheck.given` returns `Nil`.\n  - If a property does not hold for all generated values, then `qcheck.given` will panic.\n- `qcheck.small_positive_or_zero_int()` generates small integers greater than or equal to zero.\n- `n + 1 == 1 + n` is the property being tested in the first test.\n  - It should be true for all generated values.\n  - The return value of `qcheck.given` will be `Nil`, because the property does hold for all generated values.\n- `n + 1 != 1 + n` is the property being tested in the second test.\n  - It should be false for all generated values.\n  - `qcheck.given` will be panic, because the property does not hold for all generated values.\n\n### In-depth example\n\nHere is a more in-depth example. We will create a simple `Point` type write some serialization functions, and then check that the serializing round-trips.\n\nFirst here is some code to define a `Point`.\n\n```gleam\ntype Point {\n  Point(Int, Int)\n}\n\nfn make_point(x: Int, y: Int) -&gt; Point {\n  Point(x, y)\n}\n\nfn point_equal(p1: Point, p2: Point) -&gt; Bool {\n  let Point(x1, y1) = p1\n  let Point(x2, y2) = p2\n\n  x1 == x2 &amp;&amp; y1 == y2\n}\n\nfn point_to_string(point: Point) -&gt; String {\n  let Point(x, y) = point\n  &quot;(&quot; &lt;&gt; int.to_string(x) &lt;&gt; &quot; &quot; &lt;&gt; int.to_string(y) &lt;&gt; &quot;)&quot;\n}\n```\n\nNext, let&#39;s write a function that parses the string representation into a `Point`. The string representation is pretty simple, `Point(1, 2)` would be represented by the following string: `(1 2)`.\n\nHere is one possible way to parse that string representation into a `Point`. (Note that this implementation is intentionally broken for illustration.)\n\n```gleam\nfn point_of_string(string: String) -&gt; Result(Point, String) {\n  // Create the regex.\n  use re &lt;- result.try(\n    regex.from_string(&quot;\\\\((\\\\d+) (\\\\d+)\\\\)&quot;)\n    |&gt; result.map_error(string.inspect),\n  )\n\n  // Ensure there is a single match.\n  use submatches &lt;- result.try(case regex.scan(re, string) {\n    [Match(_content, submatches)] -&gt; Ok(submatches)\n    _ -&gt; Error(&quot;expected a single match&quot;)\n  })\n\n  // Ensure both submatches are present.\n  use xy &lt;- result.try(case submatches {\n    [Some(x), Some(y)] -&gt; Ok(#(x, y))\n    _ -&gt; Error(&quot;expected two submatches&quot;)\n  })\n\n  // Try to parse both x and y values as integers.\n  use xy &lt;- result.try(case int.parse(xy.0), int.parse(xy.1) {\n    Ok(x), Ok(y) -&gt; Ok(#(x, y))\n    Error(Nil), Ok(_) -&gt; Error(&quot;failed to parse x value&quot;)\n    Ok(_), Error(Nil) -&gt; Error(&quot;failed to parse y value&quot;)\n    Error(Nil), Error(Nil) -&gt; Error(&quot;failed to parse x and y values&quot;)\n  })\n\n  Ok(Point(xy.0, xy.1))\n}\n```\n\nNow we would like to test our implementation. Of course, we could make some examples and test it like so:\n\n```gleam\nimport gleeunit/should\n\npub fn roundtrip_test() {\n  let point = Point(1, 2)\n  let parsed_point = point |&gt; point_to_string |&gt; point_of_string\n\n  point_equal(point, parsed_point) |&gt; should.be_true\n}\n```\n\nThat&#39;s fine, and you can imagine taking some corner cases like putting in `0` or `-1` or the max and min values for integers on your selected target. Rather, let&#39;s think of a property to test.\n\nI mention round-tripping, but how can you write a property to test it. Something like, &quot;given a valid point, when serializing it to a string, and then deserializing that string into another point, both points should always be equal&quot;.\n\nOkay, first we need to write a generator of valid points. In this case, it isn&#39;t too interesting as any integer can be used for both `x` and `y` values of the point. So we can use `generator.map2` like so:\n\n```gleam\nfn point_generator() {\n  qcheck.map2(make_point, qcheck.int_uniform(), qcheck.int_uniform())\n}\n```\n\nAlternatively, if you prefer the `use` syntax, you could write:\n\n```gleam\nfn point_generator() {\n  use x, y &lt;- qcheck.map2(g1: qcheck.int_uniform(), g2: qcheck.int_uniform())\n\n  make_point(x, y)\n}\n```\n\nNow that we have the point generator, we can write a property test.\n\n```gleam\npub fn point_serialization_roundtripping__test() {\n  use generated_point &lt;- qcheck.given(point_generator())\n\n  let assert Ok(parsed_point) =\n    generated_point\n    |&gt; point_to_string\n    |&gt; point_of_string\n\n  point_equal(generated_point, parsed_point)\n}\n```\n\nA couple things to note here.\n\n- `qcheck.given` will &quot;fail&quot; if either the property doesn&#39;t hold (e.g., returns `False`) or if there is a panic somewhere in the property function.\n  - Either, the points are not equal, or, the deserialization returns an `Error` (because we `assert` that it is `Ok`).\n  - Either one of these cases will trigger shrinking.\n\nLet&#39;s try and run the test.\n\n```\n$ gleam test\n\n  1) examples/parsing_example_test.point_serialization_roundtripping__test: module &#39;examples@parsing_example_test&#39;\n     Failure: &lt;&lt;&quot;TestError[original_value: Point(-875333649, -1929681101); shrunk_value: Point(0, -1); shrink_steps: 31; error: Errored(dict.from_list([#(Function, \\&quot;point_serialization_roundtripping__test\\&quot;), #(Line, 74), #(Message, \\&quot;Assertion pattern match failed\\&quot;), #(Module, \\&quot;examples/parsing_example_test\\&quot;), #(Value, Error(\\&quot;expected a single match\\&quot;)), #(GleamError, LetAssert)]));]&quot;&gt;&gt;\n     stacktrace:\n       qcheck_ffi.fail\n     output:\n```\n\nThere is a failure. Now, currently, this output is pretty noisy. Here are the important parts to highlight.\n\n- `original_value: Point(-875333649, -1929681101)`\n  - This is the original counter-example that causes the test to fail.\n- `shrunk_value: Point(0, -1)`\n  - Because `qcheck` generators have integrated shrinking, that counter-example &quot;shrinks&quot; to this simpler example.\n  - The &quot;shrunk&quot; examples can help you better identify what the problem may be.\n- `Error(\\&quot;expected a single match\\&quot;))`\n  - Here is the error message that actually caused the failure.\n\nSo we see a failure with `Point(0, -1)`, which means it probably has something to do with the negative number. Also, we see that the `Error(&quot;expected a single match&quot;)` is what triggered the failure. That error comes about when `regex.scan` fails in the `point_of_string` function.\n\nGiven those two pieces of information, we can infer that the issue is in our regular expression definition: `regex.from_string(&quot;\\\\((\\\\d+) (\\\\d+)\\\\)&quot;)`. And now we may notice that we are not allowing for negative numbers in the regular expression. To fix it, change that line to the following:\n\n```gleam\n    regex.from_string(&quot;\\\\((-?\\\\d+) (-?\\\\d+)\\\\)&quot;)\n```\n\nThat is allowing an optional `-` sign in front of the integers. Now when you rerun the `gleam test`, everything passes.\n\nYou could imagine combining a property test like the one above, with a few well chosen examples to anchor everything, into a nice little test suite that exercises the serialization of points in a small amount of test code.\n\n(The full code for this example can be found in `test/examples/parsing_example_test.gleam`.)\n\n### Applicative sytle\n\nThe applicative style provides a nice interface for creating generators for custom types.\n\n```gleam\nimport qcheck\n\n/// A simple Box type with position (x, y) and dimensions (width, height).\ntype Box {\n  Box(x: Int, y: Int, w: Int, h: Int)\n}\n\nfn box_generator() {\n  // Lift the Box creating function into the Generator structure.\n  qcheck.return({\n    use x &lt;- qcheck.parameter\n    use y &lt;- qcheck.parameter\n    use w &lt;- qcheck.parameter\n    use h &lt;- qcheck.parameter\n    Box(x:, y:, w:, h:)\n  })\n  // Set the `x` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(-100, 100))\n  // Set the `y` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(-100, 100))\n  // Set the `width` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(1, 100))\n  // Set the `height` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(1, 100))\n}\n```\n\n### More examples\n\nThe [test](https://github.com/mooreryan/gleam_qcheck/tree/main/test) directory of this repository has many examples of setting up tests, using the built-in generators, and creating new generators. Until more dedicated documentation is written, the tests can provide some good info, as they exercise most of the available behavior. However, be aware that the tests will often use `use &lt;- qcheck.rescue`. This is _not_ needed in your tests--it provides a way to test the `qcheck` internals.\n\n### Integrating with testing frameworks\n\nYou don&#39;t have to do anything special to integrate `qcheck` with a testing framework like [gleeunit](https://github.com/lpil/gleeunit). The only thing required is that your testing framework of choice be able to handle panics/exceptions.\n\nYou may also be interested in [qcheck_gleeunit_utils](https://github.com/mooreryan/qcheck_gleeunit_utils) for running your tests in parallel and controlling test timeouts when using gleeunit and targeting Erlang.\n\n## Roadmap\n\nWhile `qcheck` has a lot of features needed to get started with property-based testing, there are still things that could be added or improved. See the `ROADMAP.md` for more information.\n\n## Acknowledgements\n\nVery heavily inspired by the [qcheck](https://github.com/c-cube/qcheck) and [base_quickcheck](https://github.com/janestreet/base_quickcheck) OCaml packages, and of course, the Haskell libraries from which they take inspiration.\n\n## Contributing\n\nThank you for your interest in the project!\n\n- Bug reports, feature requests, suggestions and ideas are welcomed. Please open an [issue](https://github.com/mooreryan/gleam_qcheck/issues/new/choose) to start a discussion.\n- External contributions will generally not be accepted without prior discussion.\n  - If you have an idea for a new feature, please open an issue for discussion prior to working on a pull request.\n  - Small pull requests for bug fixes, typos, or other changes with limited scope may be accepted. If in doubt, please open an issue for discussion first.\n\n## License\n\n[![license MIT or Apache\n2.0](https://img.shields.io/badge/license-MIT%20or%20Apache%202.0-blue)](https://github.com/mooreryan/gleam_qcheck)\n\nCopyright (c) 2024 Ryan M. Moore\n\nLicensed under the Apache License, Version 2.0 or the MIT license, at your option. This program may not be copied, modified, or distributed except according to those terms.\n","url":"index.html"},{"doc":"qcheck","title":"Config","content":"pub type Config {\n  Config(test_count: Int, max_retries: Int, random_seed: Seed)\n}\n Configuration for the property-based testing.\n \n - `test_count`: The number of tests to run for each property.\n - `max_retries`: The number of times to retry the tested property while \n   shrinking.\n - `random_seed`: The seed for the random generator.\nConfig(test_count: Int, max_retries: Int, random_seed: Seed)\n\n\nSynonyms:\nqcheck.Config\nqcheck Config","url":"qcheck.html#Config"},{"doc":"qcheck","title":"Generator","content":"pub type Generator(a) {\n  Generator(fn(Seed) -&gt; #(Tree(a), Seed))\n}\n `Generator(a)` is a random generator for values of type `a`.\n \n *Note:* Because it exposes the prng `Seed` type, it is likely that this type \n will become opaque in the future.\n \nGenerator(fn(Seed) -&gt; #(Tree(a), Seed))\n\n\nSynonyms:\nqcheck.Generator\nqcheck Generator","url":"qcheck.html#Generator"},{"doc":"qcheck","title":"Seed","content":"pub opaque type Seed\n An opaque type representing a seed value used to initialize random generators.\n \n\nSynonyms:\nqcheck.Seed\nqcheck Seed","url":"qcheck.html#Seed"},{"doc":"qcheck","title":"TestError","content":"pub opaque type TestError(a)\n\n\nSynonyms:\nqcheck.TestError\nqcheck TestError","url":"qcheck.html#TestError"},{"doc":"qcheck","title":"TestErrorMessage","content":"pub opaque type TestErrorMessage\n\n\nSynonyms:\nqcheck.TestErrorMessage\nqcheck TestErrorMessage","url":"qcheck.html#TestErrorMessage"},{"doc":"qcheck","title":"Tree","content":"pub type Tree(a) {\n  Tree(a, Yielder(Tree(a)))\n}\n\nTree(a, Yielder(Tree(a)))\n\n\nSynonyms:\nqcheck.Tree\nqcheck Tree","url":"qcheck.html#Tree"},{"doc":"qcheck","title":"Try","content":"pub type Try(a) {\n  NoPanic(a)\n  Panic(exception.Exception)\n}\n\nNoPanic(a)\n\n\nPanic(exception.Exception)\n\n\nSynonyms:\nqcheck.Try\nqcheck Try","url":"qcheck.html#Try"},{"doc":"qcheck","title":"apply","content":"pub fn apply(\n  f f: Generator(fn(a) -&gt; b),\n  generator x: Generator(a),\n) -&gt; Generator(b)\n `apply(f, x)` applies a function generator, `f`, and an argument generator, \n `x`, into a result generator.\n \nSynonyms:\nqcheck.apply\nqcheck apply","url":"qcheck.html#apply"},{"doc":"qcheck","title":"apply_tree","content":"pub fn apply_tree(f: Tree(fn(a) -&gt; b), x: Tree(a)) -&gt; Tree(b)\n\nSynonyms:\nqcheck.apply_tree\nqcheck apply_tree","url":"qcheck.html#apply_tree"},{"doc":"qcheck","title":"bind","content":"pub fn bind(\n  generator generator: Generator(a),\n  f f: fn(a) -&gt; Generator(b),\n) -&gt; Generator(b)\n `bind(generator, f)` generates a value of type `a` with `generator`, then \n passes that value to `f`, which uses it to generate values of type `b`.\n \nSynonyms:\nqcheck.bind\nqcheck bind","url":"qcheck.html#bind"},{"doc":"qcheck","title":"bind_tree","content":"pub fn bind_tree(tree: Tree(a), f: fn(a) -&gt; Tree(b)) -&gt; Tree(b)\n\nSynonyms:\nqcheck.bind_tree\nqcheck bind_tree","url":"qcheck.html#bind_tree"},{"doc":"qcheck","title":"bool","content":"pub fn bool() -&gt; Generator(Bool)\n `bool()` generates booleans and shrinks towards `False`.\n \nSynonyms:\nqcheck.bool\nqcheck bool","url":"qcheck.html#bool"},{"doc":"qcheck","title":"char","content":"pub fn char() -&gt; Generator(String)\n `char()` generates characters with a bias towards printable ASCII \n characters, while still hitting some edge cases.\n \nSynonyms:\nqcheck.char\nqcheck char","url":"qcheck.html#char"},{"doc":"qcheck","title":"char_alpha","content":"pub fn char_alpha() -&gt; Generator(String)\n `char_alpha()` generates alphabetic (ASCII) characters.\n \nSynonyms:\nqcheck.char_alpha\nqcheck char_alpha","url":"qcheck.html#char_alpha"},{"doc":"qcheck","title":"char_alpha_numeric","content":"pub fn char_alpha_numeric() -&gt; Generator(String)\n `char_alpha_numeric()` generates alphanumeric (ASCII) characters.\n \nSynonyms:\nqcheck.char_alpha_numeric\nqcheck char_alpha_numeric","url":"qcheck.html#char_alpha_numeric"},{"doc":"qcheck","title":"char_digit","content":"pub fn char_digit() -&gt; Generator(String)\n `char_digit()` generates digits from `0` to `9`, inclusive.\n \nSynonyms:\nqcheck.char_digit\nqcheck char_digit","url":"qcheck.html#char_digit"},{"doc":"qcheck","title":"char_from_list","content":"pub fn char_from_list(chars: List(String)) -&gt; Generator(String)\n `char_from_list(chars)` generates characters from the given list of\n characters.\n \nSynonyms:\nqcheck.char_from_list\nqcheck char_from_list","url":"qcheck.html#char_from_list"},{"doc":"qcheck","title":"char_lowercase","content":"pub fn char_lowercase() -&gt; Generator(String)\n `char_lowercase()` generates lowercase (ASCII) letters.\n \nSynonyms:\nqcheck.char_lowercase\nqcheck char_lowercase","url":"qcheck.html#char_lowercase"},{"doc":"qcheck","title":"char_print","content":"pub fn char_print() -&gt; Generator(String)\n `char_print()` generates printable ASCII characters, with a bias towards\n alphanumeric characters.\n \nSynonyms:\nqcheck.char_print\nqcheck char_print","url":"qcheck.html#char_print"},{"doc":"qcheck","title":"char_print_uniform","content":"pub fn char_print_uniform() -&gt; Generator(String)\n `char_print_uniform()` generates printable ASCII characters.\n \nSynonyms:\nqcheck.char_print_uniform\nqcheck char_print_uniform","url":"qcheck.html#char_print_uniform"},{"doc":"qcheck","title":"char_uniform","content":"pub fn char_uniform() -&gt; Generator(String)\n `char_uniform()` generates characters uniformly distributed across the \n default range.\n \nSynonyms:\nqcheck.char_uniform\nqcheck char_uniform","url":"qcheck.html#char_uniform"},{"doc":"qcheck","title":"char_uniform_inclusive","content":"pub fn char_uniform_inclusive(\n  low low: Int,\n  high high: Int,\n) -&gt; Generator(String)\n `char_uniform_inclusive(low, high)` generates &quot;characters&quot; uniformly\n distributed between `low` and `high`, inclusive.  Here, &quot;characters&quot; are \n strings of a single codepoint.\n \n *Note*: this function is slightly weird in that it takes the integer \n representation of the range of codepoints, not the strings themselves.  \n This behavior will likely change.\n \n These `char_*` functions are mainly used for setting up the string \n generators.\n \n Shrinks towards `a` when possible, but won&#39;t go outside of the range.\n \nSynonyms:\nqcheck.char_uniform_inclusive\nqcheck char_uniform_inclusive","url":"qcheck.html#char_uniform_inclusive"},{"doc":"qcheck","title":"char_uppercase","content":"pub fn char_uppercase() -&gt; Generator(String)\n `char_uppercase()` generates uppercase (ASCII) letters.\n \nSynonyms:\nqcheck.char_uppercase\nqcheck char_uppercase","url":"qcheck.html#char_uppercase"},{"doc":"qcheck","title":"char_whitespace","content":"pub fn char_whitespace() -&gt; Generator(String)\n `char_whitespace()` generates whitespace (ASCII) characters.\n \nSynonyms:\nqcheck.char_whitespace\nqcheck char_whitespace","url":"qcheck.html#char_whitespace"},{"doc":"qcheck","title":"default_config","content":"pub fn default_config() -&gt; Config\n `default()` returns the default configuration for the property-based testing.\nSynonyms:\nqcheck.default_config\nqcheck default_config","url":"qcheck.html#default_config"},{"doc":"qcheck","title":"dict_generic","content":"pub fn dict_generic(\n  key_generator key_generator: Generator(a),\n  value_generator value_generator: Generator(b),\n  max_length max_length: Int,\n) -&gt; Generator(Dict(a, b))\n `dict_generic(key_generator, value_generator, max_len)` generates dictionaries with keys\n from `key_generator` and values from `value_generator` with lengths up to `max_len`.\n \n Shrinks on size then on elements.\n \nSynonyms:\nqcheck.dict_generic\nqcheck dict_generic","url":"qcheck.html#dict_generic"},{"doc":"qcheck","title":"failwith","content":"pub fn failwith(\n  original_value original_value: a,\n  shrunk_value shrunk_value: a,\n  shrink_steps shrink_steps: Int,\n  error_msg error_msg: String,\n) -&gt; b\n\nSynonyms:\nqcheck.failwith\nqcheck failwith","url":"qcheck.html#failwith"},{"doc":"qcheck","title":"float","content":"pub fn float() -&gt; Generator(Float)\n `float()` generates floats with a bias towards smaller values and shrinks \n towards `0.0`.\n \nSynonyms:\nqcheck.float\nqcheck float","url":"qcheck.html#float"},{"doc":"qcheck","title":"float_uniform_inclusive","content":"pub fn float_uniform_inclusive(\n  low: Float,\n  high: Float,\n) -&gt; Generator(Float)\n\nSynonyms:\nqcheck.float_uniform_inclusive\nqcheck float_uniform_inclusive","url":"qcheck.html#float_uniform_inclusive"},{"doc":"qcheck","title":"from_float_weighted_generators","content":"pub fn from_float_weighted_generators(\n  generators: List(#(Float, Generator(a))),\n) -&gt; Generator(a)\n `from_float_weighted_generators(generators)` chooses a generator from a list of generators\n weighted by the given float weights, then chooses a value from that generator.\n \n You should generally prefer `from_weighted_generators` as it is much faster.\n \nSynonyms:\nqcheck.from_float_weighted_generators\nqcheck from_float_weighted_generators","url":"qcheck.html#from_float_weighted_generators"},{"doc":"qcheck","title":"from_generators","content":"pub fn from_generators(\n  generators: List(Generator(a)),\n) -&gt; Generator(a)\n `from_generators(generators)` chooses a generator from a list of generators \n weighted uniformly, then chooses a value from that generator.\n \nSynonyms:\nqcheck.from_generators\nqcheck from_generators","url":"qcheck.html#from_generators"},{"doc":"qcheck","title":"from_weighted_generators","content":"pub fn from_weighted_generators(\n  generators: List(#(Int, Generator(a))),\n) -&gt; Generator(a)\n `from_float_generators(generators)` chooses a generator from a list of generators\n weighted by the given integer weights, then chooses a value from that generator.\n \n You should generally prefer this function over \n `from_float_weighted_generators` as this function is faster.\n \nSynonyms:\nqcheck.from_weighted_generators\nqcheck from_weighted_generators","url":"qcheck.html#from_weighted_generators"},{"doc":"qcheck","title":"generate_tree","content":"pub fn generate_tree(\n  generator: Generator(a),\n  seed: Seed,\n) -&gt; #(Tree(a), Seed)\n `generate(gen, seed)` generates a value of type `a` and its shrinks using the generator `gen`.\n \n You should not use this function directly. It is for internal use only.\n \nSynonyms:\nqcheck.generate_tree\nqcheck generate_tree","url":"qcheck.html#generate_tree"},{"doc":"qcheck","title":"given","content":"pub fn given(\n  generator generator: Generator(a),\n  property property: fn(a) -&gt; Bool,\n) -&gt; Nil\n A specialized version of `run` that uses the default configuration.\n \n \nSynonyms:\nqcheck.given\nqcheck given","url":"qcheck.html#given"},{"doc":"qcheck","title":"given_result","content":"pub fn given_result(\n  generator generator: Generator(a),\n  property property: fn(a) -&gt; Result(b, c),\n) -&gt; Nil\n A specialized version of `run_result` that uses the default configuration.\n \n \nSynonyms:\nqcheck.given_result\nqcheck given_result","url":"qcheck.html#given_result"},{"doc":"qcheck","title":"int_uniform","content":"pub fn int_uniform() -&gt; Generator(Int)\n `int_uniform()` generates uniformly distributed integers across a large \n range and shrinks towards `0`.\n \n Note: this generator does not hit interesting or corner cases very often.\n \nSynonyms:\nqcheck.int_uniform\nqcheck int_uniform","url":"qcheck.html#int_uniform"},{"doc":"qcheck","title":"int_uniform_inclusive","content":"pub fn int_uniform_inclusive(\n  low low: Int,\n  high high: Int,\n) -&gt; Generator(Int)\n `int_uniform_inclusive(low, high)` generates integers uniformly distributed\n between `low` and `high`, inclusive.\n \n Shrinks towards `0`, but won&#39;t shrink outside of the range `[low, high]`.\nSynonyms:\nqcheck.int_uniform_inclusive\nqcheck int_uniform_inclusive","url":"qcheck.html#int_uniform_inclusive"},{"doc":"qcheck","title":"list_generic","content":"pub fn list_generic(\n  element_generator: Generator(a),\n  min_length min_len: Int,\n  max_length max_len: Int,\n) -&gt; Generator(List(a))\n `list_generic(element_generator, min_len, max_len)` generates lists of \n elements from `element_generator` with lengths between `min_len` and \n `max_len`, inclusive.\n  \n Shrinks first on the number of elements, then on the elements themselves.\n \nSynonyms:\nqcheck.list_generic\nqcheck list_generic","url":"qcheck.html#list_generic"},{"doc":"qcheck","title":"make_primitive_tree","content":"pub fn make_primitive_tree(\n  root x: a,\n  shrink shrink: fn(a) -&gt; Yielder(a),\n) -&gt; Tree(a)\n\nSynonyms:\nqcheck.make_primitive_tree\nqcheck make_primitive_tree","url":"qcheck.html#make_primitive_tree"},{"doc":"qcheck","title":"map","content":"pub fn map(\n  generator generator: Generator(a),\n  f f: fn(a) -&gt; b,\n) -&gt; Generator(b)\n `map(generator, f)` transforms the generator `generator` by applying `f` to \n each generated value.  Shrinks as `generator` shrinks, but with `f` applied.\n \nSynonyms:\nqcheck.map\nqcheck map","url":"qcheck.html#map"},{"doc":"qcheck","title":"map2","content":"pub fn map2(\n  f f: fn(a, b) -&gt; c,\n  g1 g1: Generator(a),\n  g2 g2: Generator(b),\n) -&gt; Generator(c)\n `map2(f, g1, g2)` transforms two generators, `g1` and `g2`, by applying `f` \n to each pair of generated values.\n \nSynonyms:\nqcheck.map2\nqcheck map2","url":"qcheck.html#map2"},{"doc":"qcheck","title":"map2_tree","content":"pub fn map2_tree(\n  f: fn(a, b) -&gt; c,\n  a: Tree(a),\n  b: Tree(b),\n) -&gt; Tree(c)\n\nSynonyms:\nqcheck.map2_tree\nqcheck map2_tree","url":"qcheck.html#map2_tree"},{"doc":"qcheck","title":"map3","content":"pub fn map3(\n  f f: fn(a, b, c) -&gt; d,\n  g1 g1: Generator(a),\n  g2 g2: Generator(b),\n  g3 g3: Generator(c),\n) -&gt; Generator(d)\n `map3(f, g1, g2, g3)` transforms three generators, `g1`, `g2`, and `g3`, by\n applying `f` to each triple of generated values.\n \nSynonyms:\nqcheck.map3\nqcheck map3","url":"qcheck.html#map3"},{"doc":"qcheck","title":"map_tree","content":"pub fn map_tree(tree: Tree(a), f: fn(a) -&gt; b) -&gt; Tree(b)\n\nSynonyms:\nqcheck.map_tree\nqcheck map_tree","url":"qcheck.html#map_tree"},{"doc":"qcheck","title":"new_test_error","content":"pub fn new_test_error(\n  original_value orig: a,\n  shrunk_value shrunk: a,\n  shrink_steps steps: Int,\n  error_msg error_msg: String,\n) -&gt; TestError(a)\n\nSynonyms:\nqcheck.new_test_error\nqcheck new_test_error","url":"qcheck.html#new_test_error"},{"doc":"qcheck","title":"nil","content":"pub fn nil() -&gt; Generator(Nil)\n `nil()` is the `Nil` generator. It always returns `Nil` and does not shrink.\n \nSynonyms:\nqcheck.nil\nqcheck nil","url":"qcheck.html#nil"},{"doc":"qcheck","title":"option","content":"pub fn option(generator: Generator(a)) -&gt; Generator(Option(a))\n `option(gen)` is an `Option` generator that uses `gen` to generate `Some` \n values.  Shrinks towards `None` then towards shrinks of `gen`.\n \nSynonyms:\nqcheck.option\nqcheck option","url":"qcheck.html#option"},{"doc":"qcheck","title":"option_tree","content":"pub fn option_tree(tree: Tree(a)) -&gt; Tree(Option(a))\n\nSynonyms:\nqcheck.option_tree\nqcheck option_tree","url":"qcheck.html#option_tree"},{"doc":"qcheck","title":"parameter","content":"pub fn parameter(f: fn(a) -&gt; b) -&gt; fn(a) -&gt; b\n `parameter(f)` is used in constructing curried functions for the applicative \n style of building generators.\n \n ### Example\n \n \n import qcheck\n \n type Box {\n   Box(x: Int, y: Int, w: Int, h: Int)\n }\n \n fn box_generator() {\n   qcheck.return({\n     use x &lt;- qcheck.parameter\n     use y &lt;- qcheck.parameter\n     use w &lt;- qcheck.parameter\n     use h &lt;- qcheck.parameter\n     Box(x:, y:, w:, h:)\n   })\n   |&gt; qcheck.apply(qcheck.int_uniform_inclusive(-100, 100))\n   |&gt; qcheck.apply(qcheck.int_uniform_inclusive(-100, 100))\n   |&gt; qcheck.apply(qcheck.int_uniform_inclusive(1, 100))\n   |&gt; qcheck.apply(qcheck.int_uniform_inclusive(1, 100))\n }\n \n pub fn parameter_example__test() {\n   use _box &lt;- qcheck.given(box_generator())\n \n   // Test some interesting property of boxes here.\n \n   // (This `True` is a standin for your property.)\n   True\n }\n \n \nSynonyms:\nqcheck.parameter\nqcheck parameter","url":"qcheck.html#parameter"},{"doc":"qcheck","title":"rescue","content":"pub fn rescue(thunk: fn() -&gt; a) -&gt; Result(a, TestErrorMessage)\n This function should only be called to rescue a function that my call\n `failwith` at some point to raise an exception.  It will likely \n raise otherwise.\nSynonyms:\nqcheck.rescue\nqcheck rescue","url":"qcheck.html#rescue"},{"doc":"qcheck","title":"rescue_error","content":"pub fn rescue_error(f: fn() -&gt; a) -&gt; Result(a, String)\n\nSynonyms:\nqcheck.rescue_error\nqcheck rescue_error","url":"qcheck.html#rescue_error"},{"doc":"qcheck","title":"return","content":"pub fn return(a: a) -&gt; Generator(a)\n `return(a)` creates a generator that always returns `a` and does not shrink.\n \nSynonyms:\nqcheck.return\nqcheck return","url":"qcheck.html#return"},{"doc":"qcheck","title":"return_tree","content":"pub fn return_tree(x: a) -&gt; Tree(a)\n\nSynonyms:\nqcheck.return_tree\nqcheck return_tree","url":"qcheck.html#return_tree"},{"doc":"qcheck","title":"run","content":"pub fn run(\n  config config: Config,\n  generator generator: Generator(a),\n  property property: fn(a) -&gt; Bool,\n) -&gt; Nil\n `run(config, generator, property)` runs the `property` function against some\n test cases generated by the `generator` function according to the specified \n `config`\n \n The `run` function returns `Nil` if the property holds (i.e., return \n `True` for all test cases), or panics if the property does not \n hold for some test case `a` (i.e., returns `False` or `panic`s).\n \n \nSynonyms:\nqcheck.run\nqcheck run","url":"qcheck.html#run"},{"doc":"qcheck","title":"run_result","content":"pub fn run_result(\n  config config: Config,\n  generator generator: Generator(a),\n  property property: fn(a) -&gt; Result(b, c),\n) -&gt; Nil\n `run_result(config, generator, property)` is like `run` but the property\n function returns a `Result` instead of a `Bool`.\n \n\nSynonyms:\nqcheck.run_result\nqcheck run_result","url":"qcheck.html#run_result"},{"doc":"qcheck","title":"seed_new","content":"pub fn seed_new(n: Int) -&gt; Seed\n `seed_new(n) creates a new seed from the given integer, `n`.\n\n ### Example\n \n Use a specific seed for the `Config`.\n \n \n let config = \n   qcheck.default_config() \n   |&gt; qcheck.with_random_seed(qcheck.seed_new(124))\n \n \nSynonyms:\nqcheck.seed_new\nqcheck seed_new","url":"qcheck.html#seed_new"},{"doc":"qcheck","title":"seed_random","content":"pub fn seed_random() -&gt; Seed\n `seed_random()` creates a new randomly-generated seed.  You can use it when\n you don&#39;t care about having specifically reproducible results.\n\n ### Example\n \n Use a random seed for the `Config`.\n \n \n let config = \n   qcheck.default_config() \n   |&gt; qcheck.with_random_seed(qcheck.seed_random())\n \n \nSynonyms:\nqcheck.seed_random\nqcheck seed_random","url":"qcheck.html#seed_random"},{"doc":"qcheck","title":"sequence_list","content":"pub fn sequence_list(l: List(Tree(a))) -&gt; Tree(List(a))\n `sequence_list(list_of_trees)` sequsences a list of trees into a tree of lists.\n \nSynonyms:\nqcheck.sequence_list\nqcheck sequence_list","url":"qcheck.html#sequence_list"},{"doc":"qcheck","title":"set_generic","content":"pub fn set_generic(\n  element_generator: Generator(a),\n  max_length max_len: Int,\n) -&gt; Generator(Set(a))\n `set_generic(element_generator, max_len)` generates sets of elements from \n `element_generator`.\n \n Shrinks first on the number of elements, then on the elements themselves.\n \nSynonyms:\nqcheck.set_generic\nqcheck set_generic","url":"qcheck.html#set_generic"},{"doc":"qcheck","title":"shrink_atomic","content":"pub fn shrink_atomic() -&gt; fn(a) -&gt; Yielder(a)\n The `atomic` shrinker treats types as atomic, and never attempts to produce\n smaller values.\nSynonyms:\nqcheck.shrink_atomic\nqcheck shrink_atomic","url":"qcheck.html#shrink_atomic"},{"doc":"qcheck","title":"shrink_float_towards","content":"pub fn shrink_float_towards(\n  destination destination: Float,\n) -&gt; fn(Float) -&gt; Yielder(Float)\n\nSynonyms:\nqcheck.shrink_float_towards\nqcheck shrink_float_towards","url":"qcheck.html#shrink_float_towards"},{"doc":"qcheck","title":"shrink_float_towards_zero","content":"pub fn shrink_float_towards_zero() -&gt; fn(Float) -&gt; Yielder(Float)\n\nSynonyms:\nqcheck.shrink_float_towards_zero\nqcheck shrink_float_towards_zero","url":"qcheck.html#shrink_float_towards_zero"},{"doc":"qcheck","title":"shrink_int_towards","content":"pub fn shrink_int_towards(\n  destination destination: Int,\n) -&gt; fn(Int) -&gt; Yielder(Int)\n\nSynonyms:\nqcheck.shrink_int_towards\nqcheck shrink_int_towards","url":"qcheck.html#shrink_int_towards"},{"doc":"qcheck","title":"shrink_int_towards_zero","content":"pub fn shrink_int_towards_zero() -&gt; fn(Int) -&gt; Yielder(Int)\n\nSynonyms:\nqcheck.shrink_int_towards_zero\nqcheck shrink_int_towards_zero","url":"qcheck.html#shrink_int_towards_zero"},{"doc":"qcheck","title":"small_positive_or_zero_int","content":"pub fn small_positive_or_zero_int() -&gt; Generator(Int)\n `small_positive_or_zero_int()` generates small integers well suited for \n modeling the sizes of sized elements like lists or strings.\n \n Smaller numbers are more likely than larger numbers.\n \n Shrinks towards `0`.\n \nSynonyms:\nqcheck.small_positive_or_zero_int\nqcheck small_positive_or_zero_int","url":"qcheck.html#small_positive_or_zero_int"},{"doc":"qcheck","title":"small_strictly_positive_int","content":"pub fn small_strictly_positive_int() -&gt; Generator(Int)\n `small_strictly_positive_int()` generates small integers strictly greater\n than `0`.\n \nSynonyms:\nqcheck.small_strictly_positive_int\nqcheck small_strictly_positive_int","url":"qcheck.html#small_strictly_positive_int"},{"doc":"qcheck","title":"string","content":"pub fn string() -&gt; Generator(String)\n `string() generates strings with the default character generator and the \n default length generator.\n \nSynonyms:\nqcheck.string\nqcheck string","url":"qcheck.html#string"},{"doc":"qcheck","title":"string_from","content":"pub fn string_from(\n  char_generator: Generator(String),\n) -&gt; Generator(String)\n `string_from(char_generator)` generates strings from the given character generator \n using the default length generator.\n \nSynonyms:\nqcheck.string_from\nqcheck string_from","url":"qcheck.html#string_from"},{"doc":"qcheck","title":"string_generic","content":"pub fn string_generic(\n  char_generator: Generator(String),\n  length_generator: Generator(Int),\n) -&gt; Generator(String)\n `string_generic(char_generator, length_generator)` generates strings with \n characters from `char_generator` and lengths from `length_generator`.\n \nSynonyms:\nqcheck.string_generic\nqcheck string_generic","url":"qcheck.html#string_generic"},{"doc":"qcheck","title":"string_non_empty","content":"pub fn string_non_empty() -&gt; Generator(String)\n `string_non_empty()` generates non-empty strings with the default character \n generator and the default length generator.\n \nSynonyms:\nqcheck.string_non_empty\nqcheck string_non_empty","url":"qcheck.html#string_non_empty"},{"doc":"qcheck","title":"string_non_empty_from","content":"pub fn string_non_empty_from(\n  char_generator: Generator(String),\n) -&gt; Generator(String)\n `string_non_empty_from(char_generator)` generates non-empty strings from the given \n character generator using the default length generator.\n \nSynonyms:\nqcheck.string_non_empty_from\nqcheck string_non_empty_from","url":"qcheck.html#string_non_empty_from"},{"doc":"qcheck","title":"string_with_length","content":"pub fn string_with_length(length: Int) -&gt; Generator(String)\n `string_with_length(length)` generates strings of the given `length` with the \n default character generator.\n \nSynonyms:\nqcheck.string_with_length\nqcheck string_with_length","url":"qcheck.html#string_with_length"},{"doc":"qcheck","title":"string_with_length_from","content":"pub fn string_with_length_from(\n  generator: Generator(String),\n  length: Int,\n) -&gt; Generator(String)\n `string_with_length_from(gen, length)` generates strings of the given \n `length` from the given generator.\n \nSynonyms:\nqcheck.string_with_length_from\nqcheck string_with_length_from","url":"qcheck.html#string_with_length_from"},{"doc":"qcheck","title":"test_error_message_original_value","content":"pub fn test_error_message_original_value(\n  msg: TestErrorMessage,\n) -&gt; String\n\nSynonyms:\nqcheck.test_error_message_original_value\nqcheck test_error_message_original_value","url":"qcheck.html#test_error_message_original_value"},{"doc":"qcheck","title":"test_error_message_shrink_steps","content":"pub fn test_error_message_shrink_steps(\n  msg: TestErrorMessage,\n) -&gt; String\n\nSynonyms:\nqcheck.test_error_message_shrink_steps\nqcheck test_error_message_shrink_steps","url":"qcheck.html#test_error_message_shrink_steps"},{"doc":"qcheck","title":"test_error_message_shrunk_value","content":"pub fn test_error_message_shrunk_value(\n  msg: TestErrorMessage,\n) -&gt; String\n\nSynonyms:\nqcheck.test_error_message_shrunk_value\nqcheck test_error_message_shrunk_value","url":"qcheck.html#test_error_message_shrunk_value"},{"doc":"qcheck","title":"tree_to_string","content":"pub fn tree_to_string(\n  tree: Tree(a),\n  a_to_string: fn(a) -&gt; String,\n) -&gt; String\n `tree_to_string(tree, element_to_string)` converts a tree into an unspecified string representation.\n \n - `element_to_string`: a function that converts individual elements of the tree to strings.\n \nSynonyms:\nqcheck.tree_to_string\nqcheck tree_to_string","url":"qcheck.html#tree_to_string"},{"doc":"qcheck","title":"tree_to_string_","content":"pub fn tree_to_string_(\n  tree: Tree(a),\n  a_to_string: fn(a) -&gt; String,\n  max_depth max_depth: Int,\n) -&gt; String\n Like `tree_to_string` but with a configurable `max_depth`.\n \nSynonyms:\nqcheck.tree_to_string_\nqcheck tree_to_string_","url":"qcheck.html#tree_to_string_"},{"doc":"qcheck","title":"try","content":"pub fn try(f: fn() -&gt; a) -&gt; Try(a)\n\nSynonyms:\nqcheck.try\nqcheck try","url":"qcheck.html#try"},{"doc":"qcheck","title":"tuple2","content":"pub fn tuple2(\n  g1: Generator(a),\n  g2: Generator(b),\n) -&gt; Generator(#(a, b))\n `tuple2(g1, g2)` generates a tuple of two values, one each from generators \n `g1` and `g2`.\n \nSynonyms:\nqcheck.tuple2\nqcheck tuple2","url":"qcheck.html#tuple2"},{"doc":"qcheck","title":"tuple3","content":"pub fn tuple3(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n) -&gt; Generator(#(a, b, c))\n `tuple3(g1, g2, g3)` generates a tuple of three values, one each from\n generators `g1`, `g2`, and `g3`.\n \nSynonyms:\nqcheck.tuple3\nqcheck tuple3","url":"qcheck.html#tuple3"},{"doc":"qcheck","title":"tuple4","content":"pub fn tuple4(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n) -&gt; Generator(#(a, b, c, d))\n `tuple4(g1, g2, g3, g4)` generates a tuple of four values, one each from\n generators `g1`, `g2`, `g3`, and `g4`.\n \nSynonyms:\nqcheck.tuple4\nqcheck tuple4","url":"qcheck.html#tuple4"},{"doc":"qcheck","title":"tuple5","content":"pub fn tuple5(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n  g5: Generator(e),\n) -&gt; Generator(#(a, b, c, d, e))\n `tuple5(g1, g2, g3, g4, g5)` generates a tuple of five values, one each from\n generators `g1`, `g2`, `g3`, `g4`, and `g5`.\n \nSynonyms:\nqcheck.tuple5\nqcheck tuple5","url":"qcheck.html#tuple5"},{"doc":"qcheck","title":"tuple6","content":"pub fn tuple6(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n  g5: Generator(e),\n  g6: Generator(f),\n) -&gt; Generator(#(a, b, c, d, e, f))\n `tuple6(g1, g2, g3, g4, g5, g6)` generates a tuple of six values, one each \n from generators `g1`, `g2`, `g3`, `g4`, `g5`, and `g6`.\n \nSynonyms:\nqcheck.tuple6\nqcheck tuple6","url":"qcheck.html#tuple6"},{"doc":"qcheck","title":"with_max_retries","content":"pub fn with_max_retries(\n  config: Config,\n  max_retries: Int,\n) -&gt; Config\n `with_max_retries()` returns a new configuration with the given max retries.\nSynonyms:\nqcheck.with_max_retries\nqcheck with_max_retries","url":"qcheck.html#with_max_retries"},{"doc":"qcheck","title":"with_random_seed","content":"pub fn with_random_seed(\n  config: Config,\n  random_seed: Seed,\n) -&gt; Config\n `with_random_seed()` returns a new configuration with the given random seed.\nSynonyms:\nqcheck.with_random_seed\nqcheck with_random_seed","url":"qcheck.html#with_random_seed"},{"doc":"qcheck","title":"with_test_count","content":"pub fn with_test_count(config: Config, test_count: Int) -&gt; Config\n `with_test_count()` returns a new configuration with the given test count.\nSynonyms:\nqcheck.with_test_count\nqcheck with_test_count","url":"qcheck.html#with_test_count"},{"doc":"qcheck","title":"qcheck","content":" QuickCheck-inspired property-based testing with integrated shrinking\n \n \n ## Overview\n \n Rather than specifying test cases manually, you describe the invariants\n that values of a given type must satisfy (&quot;properties&quot;). Then, generators\n generate lots of values (test cases) on which the properties are checked.\n Finally, if a value is found for which a given property does not hold, that\n value is &quot;shrunk&quot; in order to find an nice, informative counter-example \n that is presented to you.\n \n This module has functions for running and configuring property tests as\n well as generating random values (with shrinking) to drive those tests.\n\n For usage examples, see the project README.\n \n \n ## Running tests\n \n - [given](#given)\n - [given_result](#given_result)\n - [run](#run)\n - [run_result](#run_result)\n \n \n ## Configuring test runs\n \n - The [Config](#Config) type\n - [default_config](#default_config)\n - [with_test_count](#with_test_count)\n - [with_max_retries](#with_max_retries)\n - [with_random_seed](#with_random_seed)\n \n \n ## Generators\n \n - The [Generator](#Generator) type\n \n Here is a list of generator functions grouped by category.\n \n ### Combinators\n \n - [return](#return)\n - [parameter](#parameter)\n - [map](#map)\n - [bind](#bind)\n - [apply](#apply)\n - [map2](#map2)\n - [map3](#map3)\n - [map4](#map4)\n - [map5](#map5)\n - [map6](#map6)\n - [tuple2](#tuple2)\n - [tuple3](#tuple3)\n - [tuple4](#tuple4)\n - [tuple5](#tuple5)\n - [tuple6](#tuple6)\n - [from_generators](#from_generators)\n - [from_weighted_generators](#from_weighted_generators)\n - [from_float_weighted_generators](#from_weighted_generators)\n \n ### Ints\n \n  - [int_uniform](#int_uniform)\n  - [int_uniform_inclusive](#int_uniform_inclusive)\n  - [small_positive_or_zero_int](#small_positive_or_zero_int)\n  - [small_strictly_positive_int](#small_strictly_positive_int)\n \n ### Floats\n \n  - [float](#float)\n  - [float_uniform_inclusive](#float_uniform_inclusive)\n \n ### Characters\n \n  - [char](#char)\n  - [char_uniform_inclusive](#char_uniform_inclusive)\n  - [char_uppercase](#char_uppercase)\n  - [char_lowercase](#char_lowercase)\n  - [char_digit](#char_digit)\n  - [char_print_uniform](#char_print_uniform)\n  - [char_uniform](#char_uniform)\n  - [char_alpha](#char_alpha)\n  - [char_alpha_numeric](#char_alpha_numeric)\n  - [char_from_list](#char_from_list)\n  - [char_whitespace](#char_whitespace)\n  - [char_print](#char_print)\n \n ### Strings\n \n  - [string](#string)\n  - [string_from](#string_from)\n  - [string_non_empty](#string_non_empty)\n  - [string_with_length](#string_with_length)\n  - [string_with_length_from](#string_with_length_from)\n  - [string_non_empty_from](#string_non_empty_from)\n  - [string_generic](#string_generic)\n \n ### Lists\n \n  - [list_generic](#list_generic)\n \n ### Dicts\n \n - [dict_generic](#dict_generic)\n\n ### Sets\n \n  - [set_generic](#set_generic)\n\n ### Other\n \n - [bool](#bool)\n - [nil](#nil)\n - [option](#option)\n \n ## Trees\n \n There are functions for dealing with the [Tree](#Tree) type directly, but \n they are low-level and you should not need to use them much. \n \n - The [Tree](#Tree) type\n - [make_primitive_tree](#make_primitive_tree)\n - [return_tree](#return_tree)\n - [map_tree](#map_tree)\n - [map2_tree](#map2_tree)\n - [bind_tree](#bind_tree)\n - [apply_tree](#apply_tree)\n - [sequence_list](#sequence_list)\n - [option_tree](#option_tree)\n - [tree_to_string](#tree_to_string)\n - [tree_to_string_](#tree_to_string_)\n \n ## Seeding generators\n \n - The [Seed](#Seed) type\n - [seed_new](#seed_new)\n - [seed_random](#seed_random)\n \n ## Shrinking\n \n There are some public functions for dealing with shrinks and shrinking.\n Similar to the Tree functions, you often won&#39;t need to use these directly.\n\n - [shrink_atomic](#shrink_atomic)\n - [shrink_int_towards](#shrink_int_towards)\n - [shrink_int_towards_zero](#shrink_int_towards_zero)\n - [shrink_float_towards](#shrink_float_towards)\n - [shrink_float_towards_zero](#shrink_float_towards_zero)\n\n \n ## Notes\n \n - If something is marked as being “unspecified”, do not depend on it, as it\n   may change at any time without a major version bump. This mainly applies \n   to the various `*_to_string` functions.\n - `TestError`, `TestErrorMessage`, and association functions will likely \n   become private as they are mainly internal machinery for displaying \n   errors.\n - `failwith`, `try`, and `rescue` will also likely become private as they \n   deal with internal property test running machinery.\n \n ","url":"qcheck.html"}]);